/********************************************************************************/
/*                                                                              */
/*              bounceball.cxx                                                  */
/*                                                                              */
/*      Control methods (BounceMainInfo) for bouncing balls demo program        */
/*                                                                              */
/********************************************************************************/


#include "bounce_local.H"

#include "bounce_ball.H"
#include "bounce_context.H"


/********************************************************************************/
/*                                                                              */
/*      BounceBallInfo constructors/destructors                                 */
/*                                                                              */
/********************************************************************************/


BounceBallInfo::BounceBallInfo(Coord maxv)
{
   Integer i;

   for ( ; ; ) {
      x_vel = 2.0 * maxv * rand()/((float)RAND_MAX) - maxv;
      y_vel = 2.0 * maxv * rand()/((float)RAND_MAX) - maxv;
      if (fabs(x_vel) > MIN_VELOCITY && fabs(y_vel) > MIN_VELOCITY) break;
    }

   x_pos = y_pos = -1;
   _radius = DEFAULT_RADIUS;
   draw_shape = BOUNCE_SHAPE_SQUARE;
   bounce_type = BOUNCE_TYPE_RANDOM;
   color_set = BOUNCE_COLOR_ALL;
   line_style = BOUNCE_LINE_NONE;

   is_drawn = false;

   for ( ; ; ) {
      i = (1 << (rand()%BOUNCE_NUM_COLOR));
      if ((color_set & i) != 0) break;
    }
   cur_color = (BounceColor) i;
}



BounceBallInfo::~BounceBallInfo()                       { }




/********************************************************************************/
/*                                                                              */
/*      BounceBallInfo property update methods                                  */
/*                                                                              */
/********************************************************************************/


BounceBall
BounceBallInfo::clone(Coord maxv)
{
   BounceBallInfo * bb = new BounceBallInfo(maxv);
   bb->copyProps(this);

   return bb;
}



void
BounceBallInfo::copyProps(BounceBall bb)
{
   _radius = bb->getRadius();
   draw_shape = bb->getShape();
   bounce_type = bb->getBounce();
   color_set = bb->getColors();
   line_style = bb->getLineStyle();
}



/********************************************************************************/
/*                                                                              */
/*      BounceBallInfo property i/o methods                                     */
/*                                                                              */
/********************************************************************************/


void
BounceBallInfo::outputProps(ostream& ost)
{
   ost << _radius;
   ost << " " << draw_shape;
   ost << " " << bounce_type;
   ost << " " << color_set;
   ost << " " << line_style;
}



Bool
BounceBallInfo::inputProps(istream& ist)
{
   Integer bs,bb,bc,bl;
   Coord br;

   ist >> br >> bs >> bb >> bc >> bl;
   if (ist.fail()) return FALSE;

   _radius = br;
   draw_shape = (BounceShape) bs;
   bounce_type = (BounceType) bb;
   color_set = (BounceColor) bc;
   line_style = (BounceLine) bl;

   return TRUE;
}



/********************************************************************************/
/*                                                                              */
/*      BounceBallInfo animation methods                                        */
/*                                                                              */
/********************************************************************************/


void
BounceBallInfo::update(Integer maxx,Integer maxy,Coord maxv)
{
   if (x_pos < 0 && y_pos < 0) {
      x_pos = maxx/2;
      y_pos = maxy/2;
      return;
    }

   x_pos += x_vel;
   y_pos += y_vel;

   if (x_pos < _radius) {
      x_pos = _radius;
      x_vel = newVelocity(x_vel,1,maxv);
      changeColor();
    }
   else if (x_pos+_radius >= maxx) {
      x_pos = maxx - _radius - 1;
      x_vel = newVelocity(x_vel,-1,maxv);
      changeColor();
    }
   if (y_pos < _radius) {
      y_pos = _radius;
      y_vel = newVelocity(y_vel,1,maxv);
      changeColor();
    }
   else if (y_pos + _radius >= maxy) {
      y_pos = maxy - _radius - 1;
      y_vel = newVelocity(y_vel,-1,maxv);
      changeColor();
    }
}



Coord
BounceBallInfo::newVelocity(Coord old,Coord dir,Coord maxv) const
{
   old = fabs(old);
   if (bounce_type == BOUNCE_TYPE_RANDOM) {
      for ( ; ; ) {
         old = rand()/((float)RAND_MAX) * maxv;
         if (old >= MIN_VELOCITY) break;
       }
    }

   return old * dir;
}



void
BounceBallInfo::changeColor()
{
   Integer i = cur_color;

   for ( ; ; ) {
      i = i << 1;
      if (i >= BOUNCE_MAX_COLOR) i = 1;
      if ((color_set & i) != 0) break;
    }
   cur_color = (BounceColor) i;
}




/********************************************************************************/
/*                                                                              */
/*      BounceBallInfo drawing methods                                          */
/*                                                                              */
/********************************************************************************/


void
BounceBallInfo::undraw(BounceDrawContext ctx)
{
   if (is_drawn) draw(ctx);
   is_drawn = false;
}



void
BounceBallInfo::draw(BounceDrawContext ctx)
{
   if (x_pos < 0) return;

   ctx->setLineStyle(line_style);
   ctx->setColor(cur_color);
   ctx->setShape(draw_shape);

   ctx->draw(x_pos,y_pos,_radius);

   is_drawn = true;
}



/* end of bounceball.cxx */
