/********************************************************************************/
/*                                                                              */
/*              bouncecontext.cxx                                               */
/*                                                                              */
/*      Drawing context for bouncing balls demo program                         */
/*                                                                              */
/********************************************************************************/


#include "bounce_local.H"

#include "bounce_context.H"



/********************************************************************************/
/*                                                                              */
/*      Color Tables                                                            */
/*                                                                              */
/********************************************************************************/


static ConstText color_names[] = {
   "black", "red", "green", "yellow", "blue", "cyan", NULL
};



/********************************************************************************/
/*                                                                              */
/*      BounceDrawContextInfo constuructors/destructors                         */
/*                                                                              */
/********************************************************************************/


BounceDrawContextInfo::BounceDrawContextInfo()
{
   cur_color = BOUNCE_COLOR_NONE;
   line_style = BOUNCE_LINE_UNDEF;
   cur_shape = BOUNCE_SHAPE_SQUARE;
}



BounceDrawContextInfo::~BounceDrawContextInfo()                 { }



/********************************************************************************/
/*                                                                              */
/*      BounceDrawContextInfo window setup methods                              */
/*                                                                              */
/********************************************************************************/


void
BounceDrawContextInfo::setupWindow(BaumWidget bw)
{
   if (draw_window != BaumWindow::EMPTY) return;

   BaumColor bkg(BACK_COLOR);

   setupColors(bw);

   draw_window.create(bw);
   draw_gc.create(draw_window);
   draw_window.background(bkg);
   draw_gc.function(6);
}



/********************************************************************************/
/*                                                                              */
/*      BounceDrawContextInfo clearing methods                                  */
/*                                                                              */
/********************************************************************************/


void
BounceDrawContextInfo::clear()
{
   draw_window.clear();
}





/********************************************************************************/
/*                                                                              */
/*      Color Management methods                                                */
/*                                                                              */
/********************************************************************************/


void
BounceDrawContextInfo::setupColors(BaumWidget bw)
{
   BaumColor bcol(BACK_COLOR);
   Integer bpix = bcol.inqPixel(bw);

   for (Integer i = 0; color_names[i] != NULL; ++i) {
      BaumColor fcol = color_names[i];
      Integer fpix = fcol.inqPixel(bw);
      color_table[i].setColor(fpix^bpix);
    }

   BaumColor lcol = LINE_COLOR;
   Integer fpix = lcol.inqPixel(bw);
   line_color.setColor(fpix^bpix);
}



/********************************************************************************/
/*                                                                              */
/*      BounceDrawContextInfo property setting methods                          */
/*                                                                              */
/********************************************************************************/


void
BounceDrawContextInfo::setLineStyle(BounceLine ls)
{
   if (ls == line_style) return;

   line_style = ls;

   switch (line_style) {
      case BOUNCE_LINE_NONE :
         break;
      case BOUNCE_LINE_SOLID :
         draw_gc.lineWidth(1);
         draw_gc.lineStyle(LineSolid);
         break;
      case BOUNCE_LINE_THICK_SOLID :
         draw_gc.lineWidth(3);
         draw_gc.lineStyle(LineSolid);
         break;
      case BOUNCE_LINE_DOTTED :
         draw_gc.lineWidth(1);
         draw_gc.lineStyle(LineOnOffDash);
         break;
      case BOUNCE_LINE_THICK_DOTTED :
         draw_gc.lineWidth(3);
         draw_gc.lineStyle(LineOnOffDash);
         break;
    }
}



void
BounceDrawContextInfo::setColor(BounceColor c)
{
   Integer i;

   if (cur_color == c) return;

   cur_color = c;

   for (i = 0; i < BOUNCE_NUM_COLOR; ++i) {
      if (((1 << i) & cur_color) != 0) break;
    }
   if (i >= BOUNCE_NUM_COLOR) return;

   draw_gc.foreground(color_table[i]);
}



void
BounceDrawContextInfo::setShape(BounceShape s)
{
   cur_shape = s;
}




/********************************************************************************/
/*                                                                              */
/*      BounceDrawContextInfo drawing methods                                   */
/*                                                                              */
/********************************************************************************/


void
BounceDrawContextInfo::draw(Integer x,Integer y,Integer r)
{
   Integer lx,ty,wd,ht;

   wd = ht = (Integer) r * 2;
   lx = (Integer) (x - r);
   ty = (Integer) (y - r);

   switch (cur_shape) {
      case BOUNCE_SHAPE_CIRCLE :
         drawCircle(lx,ty,wd,ht);
         break;
      case BOUNCE_SHAPE_SQUARE :
         drawSquare(lx,ty,wd,ht);
         break;
      case BOUNCE_SHAPE_DIAMOND :
         drawDiamond(lx,ty,wd,ht);
         break;
    }
}



void
BounceDrawContextInfo::drawCircle(Integer lx,Integer ty,Integer wd,Integer ht)
{
   draw_window.fillArc(draw_gc,lx,ty,wd,ht,0*64,360*64);
   if (line_style != BOUNCE_LINE_NONE) {
      setColor(BOUNCE_COLOR_BLACK);
      draw_window.drawArc(draw_gc,lx,ty,wd,ht,0*64,360*64);
    }
}




void
BounceDrawContextInfo::drawDiamond(Integer lx,Integer ty,Integer wd,Integer ht)
{
   XPoint pts[5];

   pts[0].x = pts[2].x = lx+wd/2;
   pts[0].y = ty;
   pts[1].x = lx+wd;
   pts[1].y = pts[3].y = ty+ht/2;
   pts[2].y = ty+ht;
   pts[3].x = lx;
   pts[4] = pts[0];

   draw_window.fillPolygon(draw_gc,pts,4);
   if (line_style != BOUNCE_LINE_NONE) {
      setColor(BOUNCE_COLOR_BLACK);
      draw_window.drawLines(draw_gc,pts,5);
    }
}




void
BounceDrawContextInfo::drawSquare(Integer lx,Integer ty,Integer wd,Integer ht)
{
   draw_window.fillRectangle(draw_gc,lx,ty,wd,ht);
   if (line_style != BOUNCE_LINE_NONE) {
      setColor(BOUNCE_COLOR_BLACK);
      draw_window.drawRectangle(draw_gc,lx,ty,wd,ht);
    }
}




/* end of bouncecontext.cxx */
