/********************************************************************************/
/*                                                                              */
/*              bouncedialog.cxx                                                */
/*                                                                              */
/*      Dialog management methods for bouncing balls demo program               */
/*                                                                              */
/********************************************************************************/


#include "bounce_local.H"

#include "bounce_dialog.H"
#include "bounce_manager.H"
#include "bounce_ball.H"



/********************************************************************************/
/*                                                                              */
/*      Fallback Resources                                                      */
/*                                                                              */
/********************************************************************************/


static  BaumDefaultResource bounce_resources(
   "*numberDialog*background: wheat",
   "*velocityDialog*background: wheat",
   "*propertyDialog*background: wheat",
   "*aboutDialog*background: wheat",
   "*saveFileDialog*background: wheat",
   "*loadFileDialog*background: wheat",
   NULL
);




/********************************************************************************/
/*                                                                              */
/*      BounceDialogInfo -- generic dialog processing                           */
/*                                                                              */
/********************************************************************************/


BounceDialogInfo::BounceDialogInfo(BounceManager bm)
{
   the_manager = bm;
   ok_flag = FALSE;
}



BounceDialogInfo::~BounceDialogInfo()
{
   if (the_dialog.isDefined()) the_dialog.destroy();
}



void
BounceDialogInfo::activate()
{
   if (the_dialog.isRealized()) the_dialog.parent().raise();
   else {
      setupValues();
      the_dialog.query(this);
    }
}



Bool
BounceDialogInfo::activateModal()
{
   ok_flag = FALSE;

   setupValues();
   the_dialog.waitFor(this);

   return ok_flag;
}



void
BounceDialogInfo::setup(ConstText name,BaumWidget parent)
{
   Character buf[10240];
   ostrstream ost(buf,10240);

   getContents(ost);
   ost << ends;

   the_dialog.create(name,parent,buf);
   the_dialog.setBorder(2);
}



void
BounceDialogInfo::dialogCallback(BaumDialog& bd,int btn)
{
   if (btn == BAUM_DIALOG_ACCEPT) ok_flag = TRUE;

   if (btn == BAUM_DIALOG_ACCEPT || btn == BAUM_DIALOG_APPLY) {
      handleApply();
    }
}




/********************************************************************************/
/*                                                                              */
/*      BounceNumberDialog methods                                              */
/*                                                                              */
/********************************************************************************/


BounceNumberDialogInfo::BounceNumberDialogInfo(BounceManager bm,BaumWidget bw)
   : BounceDialogInfo(bm)
{
   setup("numberDialog",bw);
}



BounceNumberDialogInfo::~BounceNumberDialogInfo()                       { }



void
BounceNumberDialogInfo::getContents(ostream& ost)
{
   ost << "%C Number of balls to animate \n\n";
   ost << "   %0.100.1.50d Balls \n";
   ost << "%a%p%c";
}



void
BounceNumberDialogInfo::setupValues()
{
   the_dialog.setValue(0,the_manager->getNumBalls());
}



void
BounceNumberDialogInfo::handleApply()
{
   Integer ct = the_dialog.getIntValue(0);
   the_manager->setNumBalls(ct);
   the_manager->setup();
}




/********************************************************************************/
/*                                                                              */
/*      BounceVelocityDialog methods                                            */
/*                                                                              */
/********************************************************************************/


BounceVelocityDialogInfo::BounceVelocityDialogInfo(BounceManager bm,BaumWidget bw)
   : BounceDialogInfo(bm)
{
   setup("velocityDialog",bw);
}



BounceVelocityDialogInfo::~BounceVelocityDialogInfo()                   { }



void
BounceVelocityDialogInfo::getContents(ostream& ost)
{
   ost << "%C Maximum Ball Velocity \n\n";
   ost << "   %0.100.1.50g Velocity \n";
   ost << "%a%p%c";
}



void
BounceVelocityDialogInfo::setupValues()
{
   the_dialog.setValue(0,the_manager->getMaxVelocity());
}



void
BounceVelocityDialogInfo::handleApply()
{
   double vel = the_dialog.getRealValue(0);
   the_manager->setMaxVelocity(vel);
   the_manager->setup();
}




/********************************************************************************/
/*                                                                              */
/*      BouncePropertyDialog methods                                            */
/*                                                                              */
/********************************************************************************/


BouncePropertyDialogInfo::BouncePropertyDialogInfo(BounceManager bm,BaumWidget bw)
   : BounceDialogInfo(bm)
{
   the_ball = bm->protoBall();

   line_list = new BaumLayoutList(FALSE,FALSE);
   line_list->clear();
   line_list->addElement("None");
   line_list->addElement("Solid");
   line_list->addElement("Thick Solid");
   line_list->addElement("Dotted");
   line_list->addElement("Thick Dotted");

   setup("propertyDialog",bw);
}



BouncePropertyDialogInfo::~BouncePropertyDialogInfo()
{
   delete line_list;
}




void
BouncePropertyDialogInfo::getContents(ostream& ost)
{
   ost << "%C Ball Properties \n";
   ost << " Radius: %1M%0.100.1.20g \n";
   ost << " Shape: %1M%1'Circle,Square,Diamond'e \n";
   ost << " Bounce: %1M%2.0'Random'o %2.1'True'o \n";
   ost << " Lines: %1M%3.150.5l \n%H";
   ost << " Colors: %1M%'black'T%4.1'Black'f";
   ost << "%2M%'red'T%4.2'Red'f";
   ost << "%3M%'green'T%4.4'Green'f%T \n";
   ost << " %1M%'yellow'T%4.8'Yellow'f";
   ost << "%2M%'blue'T%4.16'Blue'f";
   ost << "%3M%'cyan'T%4.32'Cyan'f%T \n";

   ost << "%a%p%c";
}



void
BouncePropertyDialogInfo::setupValues()
{
   the_dialog.setValue(0,the_ball->getRadius());
   the_dialog.setValue(1,the_ball->getShape());
   the_dialog.setValue(2,the_ball->getBounce());
   the_dialog.setValue(3,line_list);
   the_dialog.setValue(4,the_ball->getColors());
   BounceLine ls = the_ball->getLineStyle();
   ConstText nm = line_list->element(ls);
   line_list->select(nm);
}



void
BouncePropertyDialogInfo::handleApply()
{
   the_ball->setRadius(the_dialog.getRealValue(0));
   the_ball->setShape((BounceShape) the_dialog.getIntValue(1));
   the_ball->setBounce((BounceType) the_dialog.getIntValue(2));
   the_ball->setColors((BounceColor) the_dialog.getIntValue(4));
   the_ball->setLineStyle((BounceLine)(line_list->selectionIndex()));

   the_manager->setup();
}




/********************************************************************************/
/*                                                                              */
/*      BounceAboutDialogInfo methods                                           */
/*                                                                              */
/********************************************************************************/


BounceAboutDialogInfo::BounceAboutDialogInfo(BounceManager bm,BaumWidget bw)
   : BounceDialogInfo(bm)
{
   setup("aboutDialog",bw);
}



BounceAboutDialogInfo::~BounceAboutDialogInfo()                         { }



void
BounceAboutDialogInfo::getContents(ostream& ost)
{
   ost << "%C Bouncing Balls Demo Program \n\n";
   ost << " This program illustrates a number of user interface features \n";
   ost << " including pull down menus and dialog boxes with various types \n";
   ost << " of controls.  It also illustrates drawing into a window and \n";
   ost << " callbacks for input and timer events. \n\n";
   ost << "%a";
}



/********************************************************************************/
/*                                                                              */
/*      BounceFileDialogInfo methods                                            */
/*                                                                              */
/********************************************************************************/


void
BounceFileDialogInfo::getContents(ostream& ost)
{
   ost << "%C Select File For " << what_info << " \n\n";
   ost << "%0q\n";
   ost << "%a%c";
}


void
BounceFileDialogInfo::setupValues()
{
   the_dialog.setValue(0,the_manager->getFileName());
}



void
BounceFileDialogInfo::handleApply()
{
   the_manager->setFileName(the_dialog.getTextValue(0));
}




/********************************************************************************/
/*                                                                              */
/*      BounceSaveFileDialogInfo methods                                        */
/*                                                                              */
/********************************************************************************/


BounceSaveFileDialogInfo::BounceSaveFileDialogInfo(BounceManager bm,BaumWidget bw)
   : BounceFileDialogInfo(bm,"Save")
{
   setup("saveFileDialog",bw);
}



BounceSaveFileDialogInfo::~BounceSaveFileDialogInfo()                   { }





/********************************************************************************/
/*                                                                              */
/*      BounceLoadFileDialogInfo methods                                        */
/*                                                                              */
/********************************************************************************/


BounceLoadFileDialogInfo::BounceLoadFileDialogInfo(BounceManager bm,BaumWidget bw)
   : BounceFileDialogInfo(bm,"Load")
{
   setup("loadFileDialog",bw);
}



BounceLoadFileDialogInfo::~BounceLoadFileDialogInfo()                   { }





/* end of bouncedialog.cxx */
