/********************************************************************************/
/*                                                                              */
/*              bouncemanager.cxx                                               */
/*                                                                              */
/*      Manager methods for bouncing balls demo program                         */
/*                                                                              */
/********************************************************************************/


#include "bounce_local.H"

#include "bounce_manager.H"
#include "bounce_ball.H"
#include "bounce_context.H"



/********************************************************************************/
/*                                                                              */
/*      BounceManagerInfo constructors/destructors                              */
/*                                                                              */
/********************************************************************************/


BounceManagerInfo::BounceManagerInfo()
{
   num_balls = DEFAULT_NUM_BALLS;
   ball_proto = new BounceBallInfo(DEFAULT_VELOCITY);
   max_velocity = DEFAULT_VELOCITY;
   do_clear = false;
}



BounceManagerInfo::~BounceManagerInfo()
{
   num_balls = 0;
   setup();
}



/********************************************************************************/
/*                                                                              */
/*      BounceManagerInfo setup methods                                         */
/*                                                                              */
/********************************************************************************/


void
BounceManagerInfo::setup()
{
   Integer ct = the_balls.size();
   Integer i;
   BounceBall bb;

   if (num_balls < 0) num_balls = 0;

   if (ct > num_balls) {
      for (i = 0; i < ct - num_balls; ++i) {
         bb = the_balls.back();
         the_balls.pop_back();
         delete bb;
       }
    }
   else if (ct < num_balls) {
      for (i = 0; i < num_balls - ct; ++i) {
         bb = ball_proto->clone(max_velocity);
         the_balls.push_back(bb);
       }
    }

   clearDraw();
}



void
BounceManagerInfo::restart()
{
   Integer ct = num_balls;

   num_balls = 0;
   setup();                                             // remove all the balls

   num_balls = ct;
   setup();
}



/********************************************************************************/
/*                                                                              */
/*      BounceManagerInfo drawing methods                                       */
/*                                                                              */
/********************************************************************************/


void
BounceManagerInfo::clearDraw()
{
   Integer i;
   Integer ct = the_balls.size();

   for (i = 0; i < ct; ++i) the_balls[i]->markUndraw();

   do_clear = true;
}




/********************************************************************************/
/*                                                                              */
/*      BounceManagerInfo animation methods                                     */
/*                                                                              */
/********************************************************************************/


void
BounceManagerInfo::nextFrame(BounceDrawContext ctx,Integer maxx,Integer maxy)
{
   Integer i;
   Integer ct = the_balls.size();

   if (do_clear) ctx->clear();
   do_clear = false;

   for (i = 0; i < ct; ++i) {
      the_balls[i]->undraw(ctx);
      the_balls[i]->update(maxx,maxy,max_velocity);
      the_balls[i]->draw(ctx);
    }
}



/********************************************************************************/
/*                                                                              */
/*      BounceManagerInfo drawing methods                                       */
/*                                                                              */
/********************************************************************************/


void
BounceManagerInfo::redraw(BounceDrawContext ctx)
{
   Integer i;
   Integer ct = the_balls.size();

   clearDraw();
   ctx->clear();
   do_clear = false;

   for (i = 0; i < ct; ++i) {
      the_balls[i]->undraw(ctx);
      the_balls[i]->draw(ctx);
    }
}




/********************************************************************************/
/*                                                                              */
/*      BounceManagerInfo load/store methods                                    */
/*                                                                              */
/********************************************************************************/


void
BounceManagerInfo::save()
{
   if (file_name.empty()) return;

   ofstream ofs(file_name.c_str());

   if (!ofs) return;

   ofs << "BALLS";
   ofs << " " << num_balls;
   ofs << " " << max_velocity;
   ofs << " ";
   ball_proto->outputProps(ofs);
   ofs << endl;

   ofs.close();
}



void
BounceManagerInfo::load()
{
   Character buf[1024];

   if (file_name.empty()) return;

   ifstream ifs(file_name.c_str());

   if (!ifs) return;

   ifs >> buf;
   if (strcmp(buf,"BALLS") == 0) {
      Integer nb;
      Coord mv;
      ifs >> nb >> mv;
      if (ball_proto->inputProps(ifs)) {
         num_balls = nb;
         max_velocity = mv;
       }
    }

   ifs.close();
}




/* end of bouncemanager.cxx */
