/********************************************************************************/
/*                                                                              */
/*              bouncewindow.cxx                                                */
/*                                                                              */
/*      Top level window management methods for bouncing balls demo program     */
/*                                                                              */
/********************************************************************************/


#include "bounce_local.H"

#include "bounce_window.H"
#include "bounce_manager.H"
#include "bounce_view.H"
#include "bounce_dialog.H"
#include "bounce_ball.H"



/********************************************************************************/
/*                                                                              */
/*      Resource definitions                                                    */
/*                                                                              */
/********************************************************************************/


static  BaumDefaultResource bounce_resources(
   "*bounceDraw.width: 400",
   "*bounceDraw.height: 400",
   NULL
);




/********************************************************************************/
/*                                                                              */
/*      Pull down menu definitions                                              */
/*                                                                              */
/********************************************************************************/


static  BaumPdmData     bounce_menus[] = {
   BaumPdmData(BAUM_PDM_MENU,"File"),
      BaumPdmData(BAUM_PDM_BTN,"About ..."),
      BaumPdmData(BAUM_PDM_BTN,"Open ..."),
      BaumPdmData(BAUM_PDM_BTN,"Save"),
      BaumPdmData(BAUM_PDM_BTN,"Save As ..."),
      BaumPdmData(BAUM_PDM_BTN,"Quit"),
   BaumPdmData(BAUM_PDM_MENU,"Animate"),
      BaumPdmData(BAUM_PDM_BTN,"Go"),
      BaumPdmData(BAUM_PDM_BTN,"Pause"),
      BaumPdmData(BAUM_PDM_BTN,"Restart"),
   BaumPdmData(BAUM_PDM_MENU,"Balls"),
      BaumPdmData(BAUM_PDM_BTN,"Set Number ..."),
      BaumPdmData(BAUM_PDM_BTN,"Set Speed ..."),
      BaumPdmData(BAUM_PDM_BTN,"Set Properties ..."),
   BaumPdmData(BAUM_PDM_END)
};




/********************************************************************************/
/*                                                                              */
/*      BounceWindowInfo constructors/destructors                               */
/*                                                                              */
/********************************************************************************/


BounceWindowInfo::BounceWindowInfo()
{
   anim_id = 0;

   number_dialog = NULL;
   velocity_dialog = NULL;
   property_dialog = NULL;
   about_dialog = NULL;
   save_dialog = NULL;
   load_dialog = NULL;

   the_manager = new BounceManagerInfo;
   draw_control = NULL;
}



BounceWindowInfo::~BounceWindowInfo()
{
   if (app_shell.isDefined()) app_shell.destroy();

   delete number_dialog;
   delete velocity_dialog;
   delete property_dialog;
   delete about_dialog;
   delete save_dialog;
   delete load_dialog;

   delete draw_control;

   delete the_manager;
}




/********************************************************************************/
/*                                                                              */
/*      BounceWindowInfo setup methods                                          */
/*                                                                              */
/********************************************************************************/


void
BounceWindowInfo::setup(BaumDisplay bd)
{
   the_display = bd;

   app_shell.createRoot(the_display);

   main_window.createManaged("bounceMain",app_shell);

   pdm_widget.createManaged("bounceMenus",main_window,bounce_menus,this);
   pdm_widget.setCallback(this);

   BaumFrameWidget frm;

   frm.shadowType(Baum::SHADOW_ETCHED_OUT);
   frm.shadowThickness(4);
   frm.createManaged("bounceFrame",main_window);

   drawing_area.createManaged("bounceDraw",frm);
   draw_control = new BounceViewInfo(the_manager);
   draw_control->setup(drawing_area);

   app_shell.realize();

   number_dialog = new BounceNumberDialogInfo(the_manager,main_window);
   velocity_dialog = new BounceVelocityDialogInfo(the_manager,main_window);
   property_dialog = new BouncePropertyDialogInfo(the_manager,main_window);
   about_dialog = new BounceAboutDialogInfo(the_manager,main_window);
   save_dialog = new BounceSaveFileDialogInfo(the_manager,main_window);
   load_dialog = new BounceLoadFileDialogInfo(the_manager,main_window);

   the_manager->setup();
}



/********************************************************************************/
/*                                                                              */
/*      BounceWindowInfo pdm callback methods                                   */
/*                                                                              */
/********************************************************************************/


void
BounceWindowInfo::pdmCallback(BaumPdmItem itm)
{
   ConstText btn = itm.name();

   if (btn == NULL) return;
   else if (strcmp(btn,"About ...") == 0) handleAbout();
   else if (strcmp(btn,"Open ...") == 0) handleOpen();
   else if (strcmp(btn,"Save") == 0) handleSave();
   else if (strcmp(btn,"Save As ...") == 0) handleSaveAs();
   else if (strcmp(btn,"Quit") == 0) handleQuit();
   else if (strcmp(btn,"Go") == 0) handleGo();
   else if (strcmp(btn,"Pause") == 0) handlePause();
   else if (strcmp(btn,"Restart") == 0) handleRestart();
   else if (strcmp(btn,"Set Number ...") == 0) handleSetNumber();
   else if (strcmp(btn,"Set Speed ...") == 0) handleSetSpeed();
   else if (strcmp(btn,"Set Properties ...") == 0) handleSetProperties();
}



/********************************************************************************/
/*                                                                              */
/*      BounceWindowInfo methods to handle window buttons                       */
/*                                                                              */
/********************************************************************************/


void
BounceWindowInfo::handleAbout()                 { about_dialog->activate(); }

void
BounceWindowInfo::handleQuit()                  { exit(0); }

void
BounceWindowInfo::handleRestart()               { the_manager->restart(); }

void
BounceWindowInfo::handleSetNumber()             { number_dialog->activate(); }

void
BounceWindowInfo::handleSetSpeed()              { velocity_dialog->activate(); }

void
BounceWindowInfo::handleSetProperties()         { property_dialog->activate(); }



/********************************************************************************/
/*                                                                              */
/*      BounceWindowInfo animation methods                                      */
/*                                                                              */
/********************************************************************************/


void
BounceWindowInfo::handleGo()
{
   if (anim_id == NULL) {
      anim_id = the_display.addTimeOut(ANIM_TIME,this);
      the_manager->setup();
    }
}



void
BounceWindowInfo::handlePause()
{
   if (anim_id != NULL) {
      the_display.removeTimeOut(anim_id);
      anim_id = NULL;
    }
}



void
BounceWindowInfo::timeoutCallback(BaumInputId id)
{
   draw_control->nextFrame();

   if (anim_id != NULL) anim_id = the_display.addTimeOut(ANIM_TIME,this);
}




/********************************************************************************/
/*                                                                              */
/*      BounceWindowInfo file management methods                                */
/*                                                                              */
/********************************************************************************/


void
BounceWindowInfo::handleOpen()
{
   if (load_dialog->activateModal()) the_manager->load();
}



void
BounceWindowInfo::handleSave()
{
   ConstText fn = the_manager->getFileName();

   if (fn == NULL || *fn == 0) {
      if (!save_dialog->activateModal()) return;
    }
   the_manager->save();
}



void
BounceWindowInfo::handleSaveAs()
{
   if (save_dialog->activateModal()) the_manager->save();
}




/* end of bouncewindow.cxx */
